@@ -24,6 +24,7 @@
 #include "arrow/testing/gtest_util.h"
 #include "arrow/testing/random.h"
 #include "arrow/util/key_value_metadata.h"
+#include "arrow/util/logging.h"
 
 namespace arrow {
 namespace compute {
@@ -31,35 +32,65 @@ namespace compute {
 const int64_t kNumItems = 1024 * 1024;
 const int64_t kFewItems = 64 * 1024;
 
-template <typename Type, typename Enable = void>
-struct GetBytesProcessed {};
+struct GetBytesProcessedVisitor {
+  const Array* arr;
+  int64_t total_bytes = 0;
 
-template <>
-struct GetBytesProcessed<BooleanType> {
-  static int64_t Get(const std::shared_ptr<Array>& arr) { return arr->length() / 8; }
-};
+  explicit GetBytesProcessedVisitor(const Array* arr) : arr(arr) {}
 
-template <typename Type>
-struct GetBytesProcessed<Type, enable_if_number<Type>> {
-  static int64_t Get(const std::shared_ptr<Array>& arr) {
-    using CType = typename Type::c_type;
-    return arr->length() * sizeof(CType);
+  Status RecurseInto(const Array* child_arr) {
+    GetBytesProcessedVisitor visitor(child_arr);
+    RETURN_NOT_OK(VisitTypeInline(*child_arr->type(), &visitor));
+    total_bytes += visitor.total_bytes;
+    return Status::OK();
   }
-};
 
-template <typename Type>
-struct GetBytesProcessed<Type, enable_if_base_binary<Type>> {
-  static int64_t Get(const std::shared_ptr<Array>& arr) {
+  Status Visit(const BooleanType& type) {
+    total_bytes = arr->length() / 8;
+    return Status::OK();
+  }
+
+  template <typename Type>
+  std::enable_if_t<is_number_type<Type>::value, Status> Visit(const Type& type) {
+    using CType = typename Type::c_type;
+    total_bytes += arr->length() * sizeof(CType);
+    return Status::OK();
+  }
+
+  template <typename Type>
+  std::enable_if_t<is_base_binary_type<Type>::value, Status> Visit(const Type& type) {
     using ArrayType = typename TypeTraits<Type>::ArrayType;
     using OffsetType = typename TypeTraits<Type>::OffsetType::c_type;
-    return arr->length() * sizeof(OffsetType) +
-           std::static_pointer_cast<ArrayType>(arr)->total_values_length();
+    total_bytes += arr->length() * sizeof(OffsetType) +
+                   internal::checked_cast<const ArrayType*>(arr)->total_values_length();
+    return Status::OK();
+  }
+
+  template <typename ArrowType>
+  enable_if_var_size_list<ArrowType, Status> Visit(const ArrowType& type) {
+    using ArrayType = typename TypeTraits<ArrowType>::ArrayType;
+    using OffsetType = typename TypeTraits<ArrowType>::OffsetType::c_type;
+
+    total_bytes += (arr->length() + 1) * sizeof(OffsetType);
+    auto child_array = internal::checked_cast<const ArrayType*>(arr)->values();
+    return RecurseInto(child_array.get());
+  }
+
+  Status Visit(const DataType& type) { return Unimplemented(type); }
+
+  Status Unimplemented(const DataType& type) {
+    return Status::NotImplemented("GetBytesProcessed not implemented for ", type);
   }
 };
 
+int64_t GetBytesProcessed(const Array& arr) {
+  GetBytesProcessedVisitor visitor(&arr);
+  ARROW_EXPECT_OK(VisitTypeInline(*arr.type(), &visitor));
+  return visitor.total_bytes;
+}
+
 template <typename Type>
-static void IfElseBench(benchmark::State& state) {
-  auto type = TypeTraits<Type>::type_singleton();
+static void IfElseBench(benchmark::State& state, const std::shared_ptr<DataType>& type) {
   using ArrayType = typename TypeTraits<Type>::ArrayType;
 
   int64_t len = state.range(0);
@@ -81,15 +112,29 @@ static void IfElseBench(benchmark::State& state) {
     ABORT_NOT_OK(IfElse(cond, left, right));
   }
 
-  state.SetBytesProcessed(state.iterations() *
-                          (GetBytesProcessed<BooleanType>::Get(cond) +
-                           GetBytesProcessed<Type>::Get(left) +
-                           GetBytesProcessed<Type>::Get(right)));
+  state.SetBytesProcessed(
+      state.iterations() *
+      (GetBytesProcessed(*cond) + GetBytesProcessed(*left) + GetBytesProcessed(*right)));
+  state.SetItemsProcessed(state.iterations() * (len - offset));
+  state.counters["offset"] = static_cast<double>(offset);
 }
 
 template <typename Type>
-static void IfElseBenchContiguous(benchmark::State& state) {
+static void IfElseBench(benchmark::State& state) {
   auto type = TypeTraits<Type>::type_singleton();
+  return IfElseBench<Type>(state, type);
+}
+
+template <typename ListType, typename ValueType>
+static void IfElseBenchList(benchmark::State& state) {
+  auto value_type = TypeTraits<ValueType>::type_singleton();
+  auto list_type = std::make_shared<ListType>(value_type);
+  return IfElseBench<ListType>(state, list_type);
+}
+
+template <typename Type>
+static void IfElseBenchContiguous(benchmark::State& state,
+                                  const std::shared_ptr<DataType>& type) {
   using ArrayType = typename TypeTraits<Type>::ArrayType;
 
   int64_t len = state.range(0);
@@ -113,10 +158,24 @@ static void IfElseBenchContiguous(benchmark::State& state) {
     ABORT_NOT_OK(IfElse(cond, left, right));
   }
 
-  state.SetBytesProcessed(state.iterations() *
-                          (GetBytesProcessed<BooleanType>::Get(cond) +
-                           GetBytesProcessed<Type>::Get(left) +
-                           GetBytesProcessed<Type>::Get(right)));
+  state.SetBytesProcessed(
+      state.iterations() *
+      (GetBytesProcessed(*cond) + GetBytesProcessed(*left) + GetBytesProcessed(*right)));
+  state.SetItemsProcessed(state.iterations() * (len - offset));
+  state.counters["offset"] = static_cast<double>(offset);
+}
+
+template <typename Type>
+static void IfElseBenchContiguous(benchmark::State& state) {
+  auto type = TypeTraits<Type>::type_singleton();
+  return IfElseBenchContiguous<Type>(state, type);
+}
+
+template <typename ListType, typename ValueType>
+static void IfElseBenchListContiguous(benchmark::State& state) {
+  auto value_type = TypeTraits<ValueType>::type_singleton();
+  auto list_type = std::make_shared<ListType>(value_type);
+  return IfElseBenchContiguous<ListType>(state, list_type);
 }
 
 static void IfElseBench64(benchmark::State& state) {
@@ -127,6 +186,14 @@ static void IfElseBench32(benchmark::State& state) {
   return IfElseBench<UInt32Type>(state);
 }
 
+static void IfElseBenchListUInt32(benchmark::State& state) {
+  return IfElseBenchList<ListType, UInt32Type>(state);
+}
+
+static void IfElseBenchListString32(benchmark::State& state) {
+  return IfElseBenchList<ListType, StringType>(state);
+}
+
 static void IfElseBenchString32(benchmark::State& state) {
   return IfElseBench<StringType>(state);
 }
@@ -143,6 +210,14 @@ static void IfElseBench32Contiguous(benchmark::State& state) {
   return IfElseBenchContiguous<UInt32Type>(state);
 }
 
+static void IfElseBenchListUInt32Contiguous(benchmark::State& state) {
+  return IfElseBenchListContiguous<ListType, UInt32Type>(state);
+}
+
+static void IfElseBenchListString32Contiguous(benchmark::State& state) {
+  return IfElseBenchListContiguous<ListType, StringType>(state);
+}
+
 static void IfElseBenchString64Contiguous(benchmark::State& state) {
   return IfElseBenchContiguous<UInt64Type>(state);
 }
@@ -184,7 +259,7 @@ static void CaseWhenBench(benchmark::State& state) {
   }
 
   // Set bytes processed to ~length of output
-  state.SetBytesProcessed(state.iterations() * GetBytesProcessed<Type>::Get(val1));
+  state.SetBytesProcessed(state.iterations() * GetBytesProcessed(*val1));
   state.SetItemsProcessed(state.iterations() * (len - offset));
 }
 
@@ -213,9 +288,9 @@ static void CaseWhenBenchList(benchmark::State& state) {
   }
 
   // Set bytes processed to ~length of output
-  state.SetBytesProcessed(state.iterations() *
-                          GetBytesProcessed<Int64Type>::Get(
-                              std::static_pointer_cast<ListArray>(val1)->values()));
+  state.SetBytesProcessed(
+      state.iterations() *
+      GetBytesProcessed(*std::static_pointer_cast<ListArray>(val1)->values()));
   state.SetItemsProcessed(state.iterations() * (len - offset));
 }
 
@@ -256,7 +331,7 @@ static void CaseWhenBenchContiguous(benchmark::State& state) {
   }
 
   // Set bytes processed to ~length of output
-  state.SetBytesProcessed(state.iterations() * GetBytesProcessed<Type>::Get(val1));
+  state.SetBytesProcessed(state.iterations() * GetBytesProcessed(*val1));
   state.SetItemsProcessed(state.iterations() * (len - offset));
 }
 
@@ -322,7 +397,7 @@ static void CoalesceBench(benchmark::State& state) {
   }
 
   state.SetBytesProcessed(state.iterations() *
-                          GetBytesProcessed<Type>::Get(arguments.front().make_array()));
+                          GetBytesProcessed(*arguments.front().make_array()));
   state.SetItemsProcessed(state.iterations() * params.length);
 }
 
@@ -343,7 +418,7 @@ static void CoalesceScalarBench(benchmark::State& state) {
   }
 
   state.SetBytesProcessed(state.iterations() *
-                          GetBytesProcessed<Type>::Get(arguments.front().make_array()));
+                          GetBytesProcessed(*arguments.front().make_array()));
   state.SetItemsProcessed(state.iterations() * params.length);
 }
 
@@ -358,8 +433,8 @@ static void CoalesceScalarStringBench(benchmark::State& state) {
     ABORT_NOT_OK(CallFunction("coalesce", arguments));
   }
 
-  state.SetBytesProcessed(state.iterations() * GetBytesProcessed<StringType>::Get(
-                                                   arguments.front().make_array()));
+  state.SetBytesProcessed(state.iterations() *
+                          GetBytesProcessed(*arguments.front().make_array()));
   state.SetItemsProcessed(state.iterations() * params.length);
 }
 
@@ -395,7 +470,7 @@ static void ChooseBench(benchmark::State& state) {
   }
 
   state.SetBytesProcessed(state.iterations() *
-                          GetBytesProcessed<Type>::Get(arguments[1].make_array()));
+                          GetBytesProcessed(*arguments[1].make_array()));
   state.SetItemsProcessed(state.iterations() * (len - offset));
 }
 
@@ -403,36 +478,41 @@ static void ChooseBench64(benchmark::State& state) {
   return ChooseBench<Int64Type>(state);
 }
 
+// IfElse: Numerics
 BENCHMARK(IfElseBench32)->Args({kNumItems, 0});
 BENCHMARK(IfElseBench64)->Args({kNumItems, 0});
-
 BENCHMARK(IfElseBench32)->Args({kNumItems, 99});
 BENCHMARK(IfElseBench64)->Args({kNumItems, 99});
-
 BENCHMARK(IfElseBench32Contiguous)->Args({kNumItems, 0});
 BENCHMARK(IfElseBench64Contiguous)->Args({kNumItems, 0});
-
 BENCHMARK(IfElseBench32Contiguous)->Args({kNumItems, 99});
 BENCHMARK(IfElseBench64Contiguous)->Args({kNumItems, 99});
 
+// IfElse: Lists
+BENCHMARK(IfElseBenchListUInt32)->Args({kNumItems, 0});
+BENCHMARK(IfElseBenchListString32)->Args({kNumItems, 0});
+BENCHMARK(IfElseBenchListUInt32Contiguous)->Args({kNumItems, 0});
+BENCHMARK(IfElseBenchListString32Contiguous)->Args({kNumItems, 0});
+
+// IfElse: Strings
 BENCHMARK(IfElseBenchString32)->Args({kNumItems, 0});
 BENCHMARK(IfElseBenchString64)->Args({kNumItems, 0});
-
 BENCHMARK(IfElseBenchString32Contiguous)->Args({kNumItems, 99});
 BENCHMARK(IfElseBenchString64Contiguous)->Args({kNumItems, 99});
 
+// CaseWhen: Numerics
 BENCHMARK(CaseWhenBench64)->Args({kNumItems, 0});
 BENCHMARK(CaseWhenBench64)->Args({kNumItems, 99});
-
 BENCHMARK(CaseWhenBench64Contiguous)->Args({kNumItems, 0});
 BENCHMARK(CaseWhenBench64Contiguous)->Args({kNumItems, 99});
 
+// CaseWhen: Lists
 BENCHMARK(CaseWhenBenchList)->Args({kFewItems, 0});
 BENCHMARK(CaseWhenBenchList)->Args({kFewItems, 99});
 
+// CaseWhen: Strings
 BENCHMARK(CaseWhenBenchString)->Args({kFewItems, 0});
 BENCHMARK(CaseWhenBenchString)->Args({kFewItems, 99});
-
 BENCHMARK(CaseWhenBenchStringContiguous)->Args({kFewItems, 0});
 BENCHMARK(CaseWhenBenchStringContiguous)->Args({kFewItems, 99});
 
