@@ -21,9 +21,7 @@
 #include "arrow/util/bit_util.h"
 #include "arrow/util/logging.h"
 
-#if defined(ARROW_HAVE_AVX2)
-
-namespace arrow::util::avx2 {
+namespace arrow::util::bit_util::avx2 {
 
 template <int bit_to_search>
 void bits_to_indexes_imp_avx2(const int num_bits, const uint8_t* bits, int* num_indexes,
@@ -211,6 +209,4 @@ bool are_all_bytes_zero_avx2(const uint8_t* bytes, uint32_t num_bytes) {
   return result_or32 == 0;
 }
 
-}  // namespace arrow::util::avx2
-
-#endif  // ARROW_HAVE_AVX2
+}  // namespace arrow::util::bit_util::avx2
